/*
     This file is part of GNUnet.
     (C) 2002, 2003 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file applications/afs/tools/gnunet-directory-emptydb.c
 * @brief tool to empty the database holding files for building directories
 * @author Christian Grothoff
 *
 * No, I don't expect that anyone but developers will find this useful...
 **/



#include "config.h"

#include <stdio.h>
#include <string.h>

#include "util/initialize.h"
#include "util/configuration.h"
#include "util/xmalloc.h"
#include "util/getopt.h"
#include "util/returnvalues.h"
#include "applications/afs/afs.h"
#include "applications/afs/encoding/directory.h"
#include "applications/common/printhelp.h"
  
static unsigned int mask = 0;

/**
 * Print a list of the options we offer.
 **/
static void printhelp() {
  printf("USAGE: gnunet-directory-emptydb [OPTIONS]\n"
	 "-a, --all                     : "
	 "remove all entries from the directory DB\n"
	 "-s, --search                  : "
	 "remove all search result entries from the directory DB\n"
	 "-i, --insert                  : "
	 "remove all insert entries from the directory DB\n"
	 "-n, --namespace               : "
	 "remove all namespace entries from the directory DB\n"
	 "-x, --directory               : "
	 "remove all directory entries from the directory DB\n");
  printHelpDefault();
}

/**
 * Perform option parsing from the command line. 
 **/
static int parseCommandLine(int argc, 
			    char * argv[]) {
  int shouldexit=0;
  int c;

  FREENONNULL(setConfigurationString("GNUNETD",
				     "LOGFILE",
				     NULL));
  while (1) {
    int option_index = 0;
    static struct GNoption long_options[] = {
      LONG_DEFAULT_OPTIONS,
      { "search",    0, 0, 's' },
      { "insert",    0, 0, 'i' },
      { "directory",    0, 0, 'x' },
      { "namespace",    0, 0, 'n' },
      { "all",    0, 0, 'a' },
      { 0,0,0,0 }
    };
    
    c = GNgetopt_long(argc,
		      argv, 
		      "vhdc:L:sixan", 
		      long_options, 
		      &option_index);    
    if (c == -1) 
      break;  /* No more flags to process */
    if (YES == parseDefaultOptions(c, GNoptarg))
      continue;    
    switch(c) {
    case 'a':
      mask = DIR_CONTEXT_ALL;
      break;
    case 's':
      mask = DIR_CONTEXT_SEARCH;
      break;
    case 'i':
      mask = DIR_CONTEXT_INSERT;
      break;
    case 'n':
      mask = DIR_CONTEXT_INSERT_SB;
      break;
    case 'x':
      mask = DIR_CONTEXT_DIRECTORY;
      break;
    case 'v': 
      printf("GNUnet v%s, gnunet-directory-emptydb v%s\n",
	     VERSION, 
	     AFS_VERSION);
      shouldexit=1;
      break;
    case 'h': 
      printhelp(); 
      shouldexit=1;
      break;
    default:
      printf("Unknown option %c. Aborting.\n"
	     "Use --help to get a list of options.\n",
	     c);
      shouldexit=1;    
    } /* end of parsing commandline */
  }
  if (GNoptind < argc) {
    printf("Invalid arguments: ");
    while (GNoptind < argc)
      printf("%s ", argv[GNoptind++]);
    printf("\nExiting.\n");
    return 1;
  }
  return shouldexit;
}

int main(int argc, 
	 char * argv[]) {
  if (SYSERR == initUtil(argc, argv, &parseCommandLine))
    return 0;

  if (mask == 0) 
    printf("You must specify which type of entries should be removed (-a, -s, -n, -i or -x).\n");
  else
    emptyDirectoryDatabase(mask);

  doneUtil();

  return 0;
}


/* end of gnunet-directory-emptydb.c */
