/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file applications/afs/tools/gnunet-search-sblock.c 
 * @brief Main function to search for SBlocks on GNUnet.
 * @author Christian Grothoff
 **/

#include "config.h"

#include <stdio.h>
#include <unistd.h>

#include "util/getopt.h"
#include "util/configuration.h"
#include "util/initialize.h"
#include "util/logging.h"
#include "util/returnvalues.h"
#include "util/xmalloc.h"

#include "applications/afs/afs.h"
#include "applications/afs/encoding/sblock.h"
#include "applications/afs/util/policy.h"
#include "applications/common/shutdown.h"
#include "applications/common/printhelp.h"
#include "util/port.h"

static HashCode160 namespace;
static HashCode160 identifier;

/**
 * Prints the usage information for this command if the user errs.
 * Aborts the program.
 **/
static void printhelp() {
  printf("USAGE: gnunet-search-sblock [OPTIONS] NAMESPACE IDENTIFIER\n"\
	 "-a LEVEL, --anonymity=LEVEL   : "\
	 "set receiver-anonymity enforcing to LEVEL\n"\
	 "-t TIMEOUT                    : "\
	 "wait TIMEOUT seconds before aborting\n");
  printHelpDefault();
}

/**
 * Parse the options, set the timeout.
 * @param argc the number of options
 * @param argv the option list (including keywords)
 * @return SYSERR if we should exit, OK otherwise
 **/
static int parseOptions(int argc,
			char ** argv) {
  int c;  

  FREENONNULL(setConfigurationString("GNUNETD",
				     "LOGFILE",
				     NULL));
  while (1) {
    int option_index = 0;
    static struct GNoption long_options[] = {
      LONG_DEFAULT_OPTIONS,
      { "anonymity", 1, 0, 'a' }, 
      { "timeout",   1, 0, 't' },
      { 0,0,0,0 }
    };    
    c = GNgetopt_long(argc,
		      argv, 
		      "a:vhdc:L:H:t:", 
		      long_options, 
		      &option_index);    
    if (c == -1) 
      break;  // No more flags to process
    if (YES == parseDefaultOptions(c, GNoptarg))
      continue;
    switch(c) {
    case 'a': {
      unsigned int receivePolicy;

      if (1 != sscanf(GNoptarg, "%ud", &receivePolicy)) {
        LOG(LOG_FAILURE,
            "FAILURE: You must pass a number to the -a option.\n");
        return -1;
      }
      setConfigurationInt("AFS",
                          "ANONYMITY-RECEIVE",
                          receivePolicy);
      break;
    }
    case 'v': 
      printf("GNUnet v%s, gnunet-search v%s\n",
	     VERSION, 
	     AFS_VERSION);
      return SYSERR;
    case 'h': 
      printhelp(); 
      return SYSERR;
    case 't': {
      unsigned int timeout;
      if (1 != sscanf(GNoptarg, "%ud", &timeout)) {
	LOG(LOG_FAILURE, 
	    "You must pass a number to the -t option.\n");
	return SYSERR;
      } else {
	setConfigurationInt("AFS",
			    "SEARCHTIMEOUT",
			    timeout);
      }
      break;
    }
    default: 
      LOG(LOG_FAILURE, 
	  "Unknown option %c. Aborting.\n"\
	  "Use --help to get a list of options.\n",
	  c);
      return SYSERR;
    } /* end of parsing commandline */
  } /* while (1) */
  if (argc - GNoptind != 2) {
    fprintf(stderr,
	    "Invalid number of arguments.  "
	    "Call with namespace and identifier (in HEX notation).\n");
    return SYSERR;
  }
  hex2hash((HexName*)argv[GNoptind++],
	   &namespace);
  hex2hash((HexName*)argv[GNoptind++],
	   &identifier);
  return OK;
}

static int searchMain(GNUNET_TCP_SOCKET * sock) {
  SBlock result;
  int ret;

  ret = searchSBlock(sock,
		     &namespace,
		     &identifier,
		     &testShutdown, 
		     NULL,
		     &result);
  if (ret == OK) {
    printSBlock(stdout,
		&result);
    run_shutdown(0); /* only one result. Abort search. */
  } else
    printf("Sorry, SBlock not found.\n");

  return ret;
}

/**
 * The main function to search for SBlocks on GNUnet.
 *
 * @param argc number of arguments from the command line
 * @param argv command line arguments
 * @return 0: ok, -1: error
 **/   
int main(int argc,
	 char ** argv) {
  GNUNET_TCP_SOCKET * sock;
  void * unused;
  pthread_t searchThread;
 
  if (SYSERR == initUtil(argc, argv, &parseOptions))
    return 0;

  sock = getClientSocket();
  initAnonymityPolicy(NULL);
  initializeShutdownHandlers();
  /* order of cron-jobs is important, thus '- cronMILLIS' */
  addCronJob((CronJob)&run_shutdown,
	     cronSECONDS * getConfigurationInt("AFS",
					       "SEARCHTIMEOUT") - cronMILLIS,
	     0, /* no need to repeat */
	     NULL);
  startCron();
  if (0 != PTHREAD_CREATE(&searchThread,
			  (PThreadMain)&searchMain,
			  sock,
			  4 * 1024))
    errexit("FATAL: failed to create search thread.\n");
  wait_for_shutdown();
  closeSocketTemporarily(sock);
  

  stopCron();
  delCronJob((CronJob)&run_shutdown,
	     0,
	     NULL);
  PTHREAD_JOIN(searchThread, &unused);

  doneAnonymityPolicy();
  releaseClientSocket(sock);
  doneUtil();
  doneShutdownHandlers();
  return 0;
}

/* end of gnunet-search-sblock.c */
