/*
     This file is part of GNUnet

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file include/applications/afs/encoding/sblock.h
 * @brief data structure SBlock
 * @author Christian Grothoff
 **/

#ifndef SBLOCK_H
#define SBLOCK_H

#include "util/tcpio.h"
#include "util/cron.h"
#include "util/hashing.h"
#include "util/hostkey.h"
#include "util/tcpio.h"
#include "util/timer.h"
#include "applications/afs/encoding/block.h"

#include <stdio.h>

typedef struct {
  /* ENCRYPTED portion (with H(keyword) == identifier): */
  /**
   * Major format version, in network byte order 
   **/
  unsigned short major_formatVersion;
  
  /**
   * Minor format version, in network byte order 
   **/
  unsigned short minor_formatVersion;

  FileIdentifier fileIdentifier; /* 48 b */
  char description[MAX_DESC_LEN]; /* 256 b */
  char filename[MAX_FILENAME_LEN/2]; /* 64 b */
  char mimetype[MAX_MIMETYPE_LEN/2]; /* 64 b */
  TIME_T creationTime;
  /**
   * An interval of 0 means no updates.  -1 means
   * sporadic updates.  Any other value specifies
   * an update period.
   */
  TIME_T updateInterval;
  HashCode160 nextIdentifier; /* N,  20 b */
  HashCode160 identifierIncrement; /* I, 20 b */

  /* NOT ENCRYTPED starting here! */
  HashCode160 identifier; /* R = H(N-I)^S */
  /* NOT SIGNED, starting here! */
  Signature signature; /* 256 b */
  PublicKey subspace; /* S = H(subspace); 264 b */
} SBlock; /* total: 1024 bytes */

/**
 * Verify that a given SBlock is well-formed.
 **/
int verifySBlock(SBlock * sb);

/**
 * Compute the "current" ID of an updateable SBlock.  Will set the ID
 * of the sblock itself for non-updateable content, the ID of the next
 * identifier for sporadically updated SBlocks and the ID computed from
 * the timing function for periodically updated SBlocks.
 *
 * @param sb the SBlock (must be in plaintext)
 * @param c the resulting current ID (set)
 **/
void computeIdAtTime(SBlock * sb,
		     TIME_T now,
		     HashCode160 * c);

/**
 * Build an SBlock.
 *
 * @param interval the update frequency (0: never, -1: sporadic)
 * @param k the key for this SBlock
 * @param n the key for the next SBlock (if updateable)
 **/
SBlock * buildSBlock(HOSTKEY pseudonym,
		     FileIdentifier * fi,
		     char * description,
		     char * filename,
		     char * mimetype,
		     TIME_T creationTime,
		     TIME_T interval,
		     HashCode160 * k,
		     HashCode160 * n);
	
/**
 * Insert the SBlock
 *
 * @return OK on success, SYSERR on error
 **/
int insertSBlock(GNUNET_TCP_SOCKET * sock,
		 SBlock * sb);

/**
 * Method to test if the receive-thread should
 * terminate.
 **/
typedef int (*TestTerminateThread)(void * context);

/**
 * Retrieve an SBlock.
 * 
 * @param sock socket to use to contact gnunetd
 * @param namespace which namespace to search
 * @param identifier ID of the SBlock in the namespace
 * @param testTerminate function to poll for abort
 * @param ttContext argument for testTerminate
 * @param result on success, the decrypted (!) SBlock is returned
 * @return OK on success, SYSERR on error
 **/
int searchSBlock(GNUNET_TCP_SOCKET * sock,
		 HashCode160 * s,
		 HashCode160 * k,
		 TestTerminateThread testTerminate,
		 void * ttContext,
		 SBlock * result);

/**
 * Print the information contained in an SBlock.
 *
 * @param sb the SBlock -- in plaintext.
 **/
void printSBlock(FILE * stream,
		 SBlock * sb);

void decryptSBlock(HashCode160 * k,
		   SBlock * in,
		   SBlock * out);


#endif
